"""
cpe
===
"""
import re

from pymongo import DESCENDING

from CveXplore.common.cpe_converters import from2to3CPE
from CveXplore.common.data_source_connection import DatasourceConnection


class Cpe(DatasourceConnection):
    """
    Cpe database object
    """

    def __init__(self, **kwargs):
        super().__init__("cpe")

        for each in kwargs:
            setattr(self, each, kwargs[each])

    def iter_cves_matching_cpe(self, vuln_prod_search: bool = False):
        """
        Generator function for iterating over cve's matching this CPE. By default the search will be made matching
        the configuration fields of the cves documents.
        """

        cpe_searchField = (
            "vulnerable_product" if vuln_prod_search else "vulnerable_configuration"
        )

        # format to cpe2.3
        cpe_string = from2to3CPE(self.cpeName)

        if cpe_string.startswith("cpe"):
            # strict search with term starting with cpe; e.g: cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:*

            remove_trailing_regex_stars = r"(?:\:|\:\:|\:\*)+$"

            cpe_regex = re.escape(re.sub(remove_trailing_regex_stars, "", cpe_string))

            cpe_regex_string = r"^{}:".format(cpe_regex)
        else:
            # more general search on same field; e.g. microsoft:windows_7
            cpe_regex_string = "{}".format(re.escape(cpe_string))

        results = self._datasource_connection.store_cves.find(
            {cpe_searchField: {"$regex": cpe_regex_string}}
        ).sort("cvss", DESCENDING)

        for each in results:
            if each is not None:
                yield each
            else:
                yield None

    def to_cve_summary(self, vuln_prod_search: bool = False) -> dict:
        """
        Method to request all cve's from the database based on this cpe object
        """
        all_cves = list(self.iter_cves_matching_cpe(vuln_prod_search=vuln_prod_search))
        data_cves = [
            d.to_dict(
                "id",
                "cvss",
                "cvss3",
                "published",
                "modified",
                "summary",
                "references",
                "status",
            )
            for d in all_cves
        ]
        data_cpe = self.to_dict()
        data_cpe["cvecount"] = len(data_cves)
        data_cpe["cves"] = data_cves
        return data_cpe

    def __repr__(self):
        """String representation of object"""
        return "<< Cpe:{} >>".format(self.id)



import click

from CveXplore.cli_cmds.cli_utils.utils import printer


@click.group(
    "find",
    invoke_without_command=True,
    help="Perform find queries on a single collection",
)
@click.option(
    "-c",
    "--collection",
    required=True,
    type=click.Choice(["capec", "cpe", "cwe", "via4", "cves"], case_sensitive=False),
    help="Collection to query",
)
@click.option("-f", "--field", required=True, help="Field to query")
@click.option("-v", "--value", required=True, help="Value to query")
@click.option("-l", "--limit", default=10, help="Query limit")
@click.option(
    "-o",
    "--output",
    default="json",
    help="Set the desired output format (defaults to json)",
    type=click.Choice(["json", "csv", "xml", "html"], case_sensitive=False),
)
@click.pass_context
def search_cmd(ctx, collection, field, value, limit, output):
    ret_list = ctx.obj["data_source"].get_single_store_entries(
        (collection, {field: value.upper()}), limit=limit
    )
    try:
        result = [result.to_dict() for result in ret_list]
    except TypeError:
        result = []

    if ctx.invoked_subcommand is None:
        printer(input_data=result, output=output)
    else:
        ctx.obj["RESULT"] = result



import pymongo


def sanitize(x: list | pymongo.cursor.Cursor):
    if isinstance(x, pymongo.cursor.Cursor):
        x = list(x)
    if type(x) == list:
        for y in x:
            sanitize(y)
    if x and "_id" in x:
        x.pop("_id")
    return x



"""
Data source connection
======================
"""
import json
import os

from CveXplore.api.connection.api_db import ApiDatabaseSource
from CveXplore.database.connection.mongo_db import MongoDBConnection
from CveXplore.objects.cvexplore_object import CveXploreObject


class DatasourceConnection(CveXploreObject):
    """
    The DatasourceConnection class handles the connection to the data source and is the base class for the database
    objects and generic database functions
    """

    # hack for documentation building
    if json.loads(os.getenv("DOC_BUILD"))["DOC_BUILD"] != "YES":
        __DATA_SOURCE_CONNECTION = (
            ApiDatabaseSource(**json.loads(os.getenv("API_CON_DETAILS")))
            if os.getenv("API_CON_DETAILS")
            else MongoDBConnection(**json.loads(os.getenv("MONGODB_CON_DETAILS")))
        )

    def to_dict(self, *print_keys: str) -> dict:
        """
        Method to convert the entire object to a dictionary
        """

        if len(print_keys) != 0:
            full_dict = {
                k: v
                for (k, v) in self.__dict__.items()
                if not k.startswith("_") and k in print_keys
            }
        else:
            full_dict = {
                k: v for (k, v) in self.__dict__.items() if not k.startswith("_")
            }

        return full_dict

    def __init__(self, collection: str):
        """
        Create a DatasourceConnection object
        """
        super().__init__()
        self.__collection = collection

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return self.__dict__ != other.__dict__

    @property
    def _datasource_connection(self):
        return DatasourceConnection.__DATA_SOURCE_CONNECTION

    @property
    def _datasource_collection_connection(self):
        return getattr(
            DatasourceConnection.__DATA_SOURCE_CONNECTION,
            "store_{}".format(self.__collection),
        )

    @property
    def _collection(self):
        return self.__collection
